var spjs_fix_form_url = {
    "version": "1.0.0.2",
    "date": "July 31, 2020",
    "data": {
        "updatedListCounter": 0,
        "listsToUpdateCounter": 0
    },
    "getFormDigest": function () {
        var r = _spPageContextInfo.formDigestValue;
        if (r === undefined) {
            r = jQuery("#__REQUESTDIGEST").val();
        }
        return r;
    },
    "getAllLists": function (baseUrl) {
        var d = jQuery.Deferred();
        jQuery.ajax({
            "url": baseUrl + "/_api/Web/Lists?$select=Id,Title,Rootfolder/Name&$expand=Rootfolder",
            "method": "GET",
            "headers": {
                "accept": "application/json; odata=verbose",
                "content-type": "application/json;odata=verbose",
                "X-RequestDigest": spjs_fix_form_url.getFormDigest()
            },
            "success": function (data) {
                var listRootfolderToList = {};
                jQuery.each(data.d.results, function (i, o) {
                    listRootfolderToList[o.RootFolder.Name.toLowerCase()] = { "title": o.Title, "id": o.Id };
                });
                d.resolve(listRootfolderToList);
            },
            "error": function (err) {
                d.reject(err);
            }
        });
        return d.promise();
    },
    "setContentTypeFormUrl": function (siteBaseUrl, list) {
        jQuery.ajax({
            "url": siteBaseUrl + "/_api/web/lists/getbyid('" + list.id + "')/ContentTypes",
            "type": "GET",
            "headers": {
                "accept": "application/json;odata=verbose",
                "content-type": "application/json;odata=verbose"
            },
            "success": function (data) {
                var updArr = [];
                jQuery.each(data.d.results, function (i, ct) {
                    if (ct.Sealed) {
                        return;
                    }
                    var idSubstr = ct.StringId.substring(0, 6);
                    var upd = false;
                    var d = {
                        '__metadata': { 'type': 'SP.ContentType' }
                    };
                    switch (idSubstr) {
                        case "0x0100": // Item
                        case "0x0102": // Event
                        case "0x0103": // Issue
                        case "0x0104": // Announcement
                        case "0x0105": // Link
                        case "0x0106": // Contact
                        case "0x0107": // Message
                        case "0x0108": // Task
                        case "0x0110": // Post
                        case "0x0111": // Comment
                            if (ct.NewFormUrl !== ct.Scope + "/NewForm.aspx") {
                                upd = true;
                                d.NewFormUrl = ct.Scope + "/NewForm.aspx";
                            }
                            if (ct.DisplayFormUrl !== ct.Scope + "/DispForm.aspx") {
                                upd = true;
                                d.DisplayFormUrl = ct.Scope + "/DispForm.aspx";
                            }
                            if (ct.EditFormUrl !== ct.Scope + "/EditForm.aspx") {
                                upd = true;
                                d.EditFormUrl = ct.Scope + "/EditForm.aspx";
                            }
                            break;
                        case "0x0101": // Document
                            if (ct.DisplayFormUrl !== ct.Scope + "/Forms/DispForm.aspx") {
                                upd = true;
                                d.DisplayFormUrl = ct.Scope + "/Forms/DispForm.aspx";
                            }
                            if (ct.EditFormUrl !== ct.Scope + "/Forms/EditForm.aspx") {
                                upd = true;
                                d.EditFormUrl = ct.Scope + "/Forms/EditForm.aspx";
                            }
                            break;
                    }
                    if (upd) {
                        updArr.push({ "d": d, "siteBaseUrl": siteBaseUrl, "list": list, "ctId": ct.StringId, "ctName": ct.Name });
                    } else {
                        console.log("- " + list.title + " (" + ct.Name + ") is already updated.");
                    }
                });
                spjs_fix_form_url.doUpdateCTUrl(updArr, 0);
            },
            "error": function (err) {
                console.log(err);
            }
        });
    },
    "doUpdateCTUrl": function (arr, index) {
        var arg = arr[index];
        if (arg !== undefined) {
            jQuery.ajax({
                "url": arg.siteBaseUrl + "/_api/web/lists/getbyid('" + arg.list.id + "')/ContentTypes('" + arg.ctId + "')",
                "type": "POST",
                "data": JSON.stringify(arg.d),
                "headers": {
                    "IF-MATCH": "*",
                    "X-HTTP-Method": "MERGE",
                    "accept": "application/json;odata=verbose",
                    "content-type": "application/json;odata=verbose",
                    "X-RequestDigest": spjs_fix_form_url.getFormDigest(),
                },
                "success": function () {
                    console.log("Updated " + arg.list.title + " (" + arg.ctName + ")");
                    index += 1;
                    spjs_fix_form_url.doUpdateCTUrl(arr, index);
                },
                "error": function (err) {
                    console.log("****** An error occurred - see details below:");
                    console.log(err);
                }
            });
        } else {
            // Done
            spjs_fix_form_url.checkProgress();
        }
    },
    "checkProgress": function () {
        spjs_fix_form_url.data.updatedListCounter += 1;
        if (spjs_fix_form_url.data.updatedListCounter === spjs_fix_form_url.data.listsToUpdateCounter) {
            console.log("*** Done with all lists in this site ***");
        }
    },
    "init": function () {
        if (typeof jQuery === "undefined") {
            setTimeout(function () {
                alert("This script requires jQuery to run. Please go to one of your DFFS enabled lists by typing in /Lists/YourList/NewForm.aspx in the browser address field and paste it in the console.");
            }, 100);
        } else {
            spjs_fix_form_url.getAllLists(_spPageContextInfo.webAbsoluteUrl).done(function (listRootfolderToList) {
                jQuery.ajax({
                    "url": _spPageContextInfo.webAbsoluteUrl + "/_api/Web/Lists/getbytitle('SPJS-DynamicFormsForSharePoint')/items?$select=Title&$top=5000",
                    "method": "GET",
                    "headers": {
                        "accept": "application/json; odata=verbose",
                        "content-type": "application/json;odata=verbose",
                        "X-RequestDigest": spjs_fix_form_url.getFormDigest()
                    },
                    "success": function (data) {
                        var dup = {}, allConfiguratedLists = [];
                        jQuery.each(data.d.results, function (i, item) {
                            var listRootfolder = item.Title.indexOf("/lists/") === 0 ? item.Title.split("/")[2] : item.Title.split("/")[1], list = listRootfolderToList[listRootfolder];
                            if (listRootfolder !== undefined && list !== undefined) {
                                if (dup[listRootfolder] === undefined) {
                                    dup[listRootfolder] = true;
                                    allConfiguratedLists.push({ "rootfolder": listRootfolder, "id": list.id, "title": list.title });
                                }
                            }
                        });
                        spjs_fix_form_url.data.listsToUpdateCounter = allConfiguratedLists.length;
                        jQuery.each(allConfiguratedLists, function (i, list) {
                            spjs_fix_form_url.setContentTypeFormUrl(_spPageContextInfo.webAbsoluteUrl, list);
                        });
                    },
                    "error": function (err) {
                        console.log(err);
                    }
                });
            });
        }
    }
};

spjs_fix_form_url.init();
// Clear console
console.clear();